package betterquesting.api2.cache;

import betterquesting.api.api.ApiReference;
import betterquesting.api.api.QuestingAPI;
import betterquesting.api.enums.EnumQuestVisibility;
import betterquesting.api.properties.NativeProps;
import betterquesting.api.questing.IQuest;
import betterquesting.api.questing.IQuestDatabase;
import betterquesting.api.storage.IQuestSettings;
import betterquesting.api2.storage.DBEntry;
import betterquesting.network.handlers.NetCacheSync;
import betterquesting.questing.QuestDatabase;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;
import java.util.UUID;
import javax.annotation.Nonnull;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.World;
import net.minecraftforge.common.IExtendedEntityProperties;

/* loaded from: betterquesting_api2_cache.jar:betterquesting/api2/cache/QuestCache.class */
public class QuestCache implements IExtendedEntityProperties {
    public static final ResourceLocation LOC_QUEST_CACHE = new ResourceLocation("betterquesting", "quest_cache");
    private final TreeSet<Integer> visibleQuests = new TreeSet<>();
    private final TreeSet<Integer> activeQuests = new TreeSet<>();
    private final TreeSet<QResetTime> resetSchedule = new TreeSet<>((o1, o2) -> {
        if (o1.questID == o2.questID) {
            return 0;
        }
        return Long.compare(o1.time, o2.time);
    });
    private final TreeSet<Integer> autoClaims = new TreeSet<>();
    private final TreeSet<Integer> markedDirty = new TreeSet<>();

    public void init(Entity entity, World world) {
    }

    public synchronized int[] getActiveQuests() {
        int i = 0;
        int[] aryAct = new int[this.activeQuests.size()];
        Iterator<Integer> it = this.activeQuests.iterator();
        while (it.hasNext()) {
            Integer q = it.next();
            int i2 = i;
            i++;
            aryAct[i2] = q.intValue();
        }
        return aryAct;
    }

    public synchronized int[] getVisibleQuests() {
        int i = 0;
        int[] aryVis = new int[this.visibleQuests.size()];
        Iterator<Integer> it = this.visibleQuests.iterator();
        while (it.hasNext()) {
            Integer q = it.next();
            int i2 = i;
            i++;
            aryVis[i2] = q.intValue();
        }
        return aryVis;
    }

    public synchronized int[] getPendingAutoClaims() {
        int i = 0;
        int[] aryAC = new int[this.autoClaims.size()];
        Iterator<Integer> it = this.autoClaims.iterator();
        while (it.hasNext()) {
            Integer q = it.next();
            int i2 = i;
            i++;
            aryAC[i2] = q.intValue();
        }
        return aryAC;
    }

    public synchronized QResetTime[] getScheduledResets() {
        return (QResetTime[]) this.resetSchedule.toArray(new QResetTime[0]);
    }

    public synchronized void markQuestDirty(int questID) {
        if (questID < 0) {
            return;
        }
        this.markedDirty.add(Integer.valueOf(questID));
    }

    public synchronized void markQuestClean(int questID) {
        if (questID < 0) {
            return;
        }
        this.markedDirty.remove(Integer.valueOf(questID));
    }

    public synchronized void cleanAllQuests() {
        this.markedDirty.clear();
    }

    public synchronized int[] getDirtyQuests() {
        int i = 0;
        int[] aryMD = new int[this.markedDirty.size()];
        Iterator<Integer> it = this.markedDirty.iterator();
        while (it.hasNext()) {
            Integer q = it.next();
            int i2 = i;
            i++;
            aryMD[i2] = q.intValue();
        }
        return aryMD;
    }

    public synchronized void updateCache(@Nonnull EntityPlayer player) {
        UUID uuid = QuestingAPI.getQuestingUUID(player);
        List<DBEntry<IQuest>> questDB = ((IQuestDatabase) QuestingAPI.getAPI(ApiReference.QUEST_DB)).getEntries();
        List<Integer> tmpVisible = new ArrayList<>();
        List<Integer> tmpActive = new ArrayList<>();
        List<QResetTime> tmpReset = new ArrayList<>();
        List<Integer> tmpAutoClaim = new ArrayList<>();
        for (DBEntry<IQuest> entry : questDB) {
            if (((IQuest) entry.getValue()).isUnlocked(uuid) || ((Boolean) ((IQuest) entry.getValue()).getProperty(NativeProps.LOCKED_PROGRESS)).booleanValue()) {
                int repeat = ((Integer) ((IQuest) entry.getValue()).getProperty(NativeProps.REPEAT_TIME)).intValue();
                NBTTagCompound ue = ((IQuest) entry.getValue()).getCompletionInfo(uuid);
                if ((ue == null && ((IQuest) entry.getValue()).getTasks().size() <= 0) || ((IQuest) entry.getValue()).canSubmit(player)) {
                    tmpActive.add(Integer.valueOf(entry.getID()));
                } else if (ue != null) {
                    if (repeat >= 0 && ((IQuest) entry.getValue()).hasClaimed(uuid)) {
                        long altTime = ue.func_74763_f("timestamp");
                        if (repeat > 1 && !((Boolean) ((IQuest) entry.getValue()).getProperty(NativeProps.REPEAT_REL)).booleanValue()) {
                            altTime -= altTime % repeat;
                        }
                        tmpReset.add(new QResetTime(entry.getID(), altTime + (repeat * 50)));
                    }
                    if (!((IQuest) entry.getValue()).hasClaimed(uuid) && ((Boolean) ((IQuest) entry.getValue()).getProperty(NativeProps.AUTO_CLAIM)).booleanValue()) {
                        tmpAutoClaim.add(Integer.valueOf(entry.getID()));
                    }
                }
            }
            if (isQuestShown((IQuest) entry.getValue(), uuid, player)) {
                tmpVisible.add(Integer.valueOf(entry.getID()));
            }
        }
        this.visibleQuests.clear();
        this.visibleQuests.addAll(tmpVisible);
        this.activeQuests.clear();
        this.activeQuests.addAll(tmpActive);
        this.resetSchedule.clear();
        this.resetSchedule.addAll(tmpReset);
        this.autoClaims.clear();
        this.autoClaims.addAll(tmpAutoClaim);
        if (player instanceof EntityPlayerMP) {
            NetCacheSync.sendSync((EntityPlayerMP) player);
        }
    }

    public synchronized void saveNBTData(NBTTagCompound tags) {
        tags.func_74783_a("visibleQuests", getVisibleQuests());
        tags.func_74783_a("activeQuests", getActiveQuests());
        tags.func_74783_a("autoClaims", getPendingAutoClaims());
        tags.func_74783_a("markedDirty", getDirtyQuests());
        NBTTagList tagSchedule = new NBTTagList();
        for (QResetTime entry : getScheduledResets()) {
            NBTTagCompound tagEntry = new NBTTagCompound();
            tagEntry.func_74768_a("quest", entry.questID);
            tagEntry.func_74772_a("time", entry.time);
            tagSchedule.func_74742_a(tagEntry);
        }
        tags.func_74782_a("resetSchedule", tagSchedule);
    }

    public synchronized void loadNBTData(NBTTagCompound nbt) {
        this.visibleQuests.clear();
        this.activeQuests.clear();
        this.resetSchedule.clear();
        this.autoClaims.clear();
        this.markedDirty.clear();
        for (int i : nbt.func_74759_k("visibleQuests")) {
            this.visibleQuests.add(Integer.valueOf(i));
        }
        for (int i2 : nbt.func_74759_k("activeQuests")) {
            this.activeQuests.add(Integer.valueOf(i2));
        }
        for (int i3 : nbt.func_74759_k("autoClaims")) {
            this.autoClaims.add(Integer.valueOf(i3));
        }
        for (int i4 : nbt.func_74759_k("markedDirty")) {
            this.markedDirty.add(Integer.valueOf(i4));
        }
        NBTTagList tagList = nbt.func_150295_c("resetSchedule", 10);
        for (int i5 = 0; i5 < tagList.func_74745_c(); i5++) {
            NBTTagCompound tagEntry = tagList.func_150305_b(i5);
            if (tagEntry.func_150297_b("quest", 99)) {
                this.resetSchedule.add(new QResetTime(tagEntry.func_74762_e("quest"), tagEntry.func_74763_f("time")));
            }
        }
    }

    /* loaded from: betterquesting_api2_cache.jar:betterquesting/api2/cache/QuestCache$QResetTime.class */
    public class QResetTime implements Comparable<QResetTime> {
        public final int questID;
        public final long time;

        private QResetTime(int questID, long time) {
            this.questID = questID;
            this.time = time;
        }

        @Override // java.lang.Comparable
        public int compareTo(QResetTime o) {
            return Long.compare(o.time, this.time);
        }

        public boolean equals(Object o) {
            return (o instanceof QResetTime) && ((QResetTime) o).questID == this.questID;
        }
    }

    public static boolean isQuestShown(IQuest quest, UUID uuid, EntityPlayer player) {
        if (quest == null || uuid == null) {
            return false;
        }
        EnumQuestVisibility vis = (EnumQuestVisibility) quest.getProperty(NativeProps.VISIBILITY);
        if (((IQuestSettings) QuestingAPI.getAPI(ApiReference.SETTINGS)).canUserEdit(player) || vis == EnumQuestVisibility.ALWAYS) {
            return true;
        }
        if (vis == EnumQuestVisibility.HIDDEN) {
            return false;
        }
        if (vis == EnumQuestVisibility.UNLOCKED) {
            return quest.isComplete(uuid) || quest.isUnlocked(uuid);
        }
        if (vis == EnumQuestVisibility.NORMAL) {
            if (quest.isComplete(uuid) || quest.isUnlocked(uuid)) {
                return true;
            }
            Iterator it = QuestDatabase.INSTANCE.bulkLookup(quest.getRequirements()).iterator();
            while (it.hasNext()) {
                if (!((IQuest) ((DBEntry) it.next()).getValue()).isUnlocked(uuid)) {
                    return false;
                }
            }
            return true;
        }
        if (vis == EnumQuestVisibility.COMPLETED) {
            return quest.isComplete(uuid);
        }
        if (vis != EnumQuestVisibility.CHAIN || quest.getRequirements().length <= 0) {
            return true;
        }
        for (DBEntry<IQuest> q : QuestDatabase.INSTANCE.bulkLookup(quest.getRequirements())) {
            if (q == null || isQuestShown((IQuest) q.getValue(), uuid, player)) {
                return true;
            }
        }
        return false;
    }
}
